@@ -29,6 +29,7 @@
 
 #include <vinagre/vinagre-debug.h>
 #include <vinagre/vinagre-prefs.h>
+#include <vinagre/vinagre-cache-prefs.h>
 
 #include "vinagre-vnc-plugin.h"
 #include "vinagre-vnc-connection.h"
@@ -317,6 +318,7 @@ impl_new_connection_from_file (VinagrePlugin *plugin,
       if (shared == 0 || shared == 1)
 	vinagre_vnc_connection_set_shared (VINAGRE_VNC_CONNECTION (conn), shared);
       else
+        /* Translators: 'shared' here is a VNC protocol specific flag. You can translate it, but I think it's better to let it untranslated */
 	g_message (_("Bad value for 'shared' flag: %d. It is supposed to be 0 or 1. Ignoring it."), shared);
     }
 
@@ -351,14 +353,22 @@ ssh_tunnel_check_toggled_cb (GtkToggleButton *button, GObject *box)
     gtk_entry_set_text (GTK_ENTRY (ssh_host_entry), "");
 }
 
+static void
+scaling_check_toggled_cb (GtkToggleButton *button, GObject *box)
+{
+  gboolean active = gtk_toggle_button_get_active (button);
+  GtkWidget *ratio = g_object_get_data (G_OBJECT (box), "ratio");
+
+  gtk_widget_set_sensitive (ratio, active);
+}
+
 static GtkWidget *
 impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
 {
-  GtkWidget *box, *check, *label, *combo, *depth_box, *ssh_box, *ssh_host_entry;
+  GtkWidget *box, *check, *label, *combo, *box2, *ssh_host_entry;
   GtkTable  *table;
-  gchar     *str;
-  gint      depth_profile;
-  const gchar *ssh_host = NULL;
+  gchar     *str, *ssh_host;
+  gboolean has_conn = VINAGRE_IS_VNC_CONNECTION (conn), active;
 
   box = gtk_vbox_new (FALSE, 0);
 
@@ -370,41 +380,58 @@ impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
   gtk_misc_set_padding (GTK_MISC (label), 0, 6);
   gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
 
-  table = GTK_TABLE (gtk_table_new (5, 2, FALSE));
+  table = GTK_TABLE (gtk_table_new (6, 2, FALSE));
   label = gtk_label_new ("  ");
   gtk_table_attach (table, label, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 
+  /* View only check button */
   check = gtk_check_button_new_with_mnemonic (_("_View only"));
   g_object_set_data (G_OBJECT (box), "view_only", check);
   gtk_table_attach_defaults (table, check, 1, 2, 0, 1);
-  if (VINAGRE_IS_VNC_CONNECTION (conn))
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
-				  vinagre_vnc_connection_get_view_only (VINAGRE_VNC_CONNECTION (conn)));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
+				has_conn ? vinagre_vnc_connection_get_view_only (VINAGRE_VNC_CONNECTION (conn))
+				: vinagre_cache_prefs_get_boolean ("vnc-connection", "view-only", FALSE));
 
+  /* Scaling check button */
   check = gtk_check_button_new_with_mnemonic (_("_Scaling"));
   g_object_set_data (G_OBJECT (box), "scaling", check);
   gtk_table_attach_defaults (table, check, 1, 2, 1, 2);
-  if (VINAGRE_IS_VNC_CONNECTION (conn))
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
-				  vinagre_vnc_connection_get_scaling (VINAGRE_VNC_CONNECTION (conn)));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
+				has_conn ? vinagre_vnc_connection_get_scaling (VINAGRE_VNC_CONNECTION (conn))
+				: vinagre_cache_prefs_get_boolean ("vnc-connection", "scaling", FALSE));
+  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
+  g_signal_connect (check,
+		    "toggled",
+		    G_CALLBACK (scaling_check_toggled_cb),
+		    box);
 
+  /* Keep ratio check button */
+  box2 = gtk_hbox_new (FALSE, 4);
+  label = gtk_label_new ("   ");
+  gtk_box_pack_start (GTK_BOX (box2), GTK_WIDGET (label), FALSE, FALSE, 0);
+  check = gtk_check_button_new_with_mnemonic (_("_Keep aspect ratio"));
+  gtk_box_pack_start (GTK_BOX (box2), check, TRUE, TRUE, 0);
+  g_object_set_data (G_OBJECT (box), "ratio", check);
+  gtk_table_attach_defaults (table, box2, 1, 2, 2, 3);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
+				has_conn ? vinagre_vnc_connection_get_keep_ratio (VINAGRE_VNC_CONNECTION (conn))
+				: vinagre_cache_prefs_get_boolean ("vnc-connection", "keep-ratio", TRUE));
+  gtk_widget_set_sensitive (check, active);
+
+  /* JPEG Compression check button */
   check = gtk_check_button_new_with_mnemonic (_("_Use JPEG Compression"));
-  gtk_widget_set_tooltip_text (check, _("This might not work on all sort of VNC servers"));
+  gtk_widget_set_tooltip_text (check, _("This might not work on all VNC servers"));
   g_object_set_data (G_OBJECT (box), "lossy", check);
-  gtk_table_attach_defaults (table, check, 1, 2, 2, 3);
-  if (VINAGRE_IS_VNC_CONNECTION (conn))
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
-				  vinagre_vnc_connection_get_lossy_encoding (VINAGRE_VNC_CONNECTION (conn)));
+  gtk_table_attach_defaults (table, check, 1, 2, 3, 4);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
+				has_conn ? vinagre_vnc_connection_get_lossy_encoding (VINAGRE_VNC_CONNECTION (conn))
+				: vinagre_cache_prefs_get_boolean ("vnc-connection", "lossy-encoding", FALSE));
 
-  depth_box = gtk_hbox_new (FALSE, 4);
+  /* Depth color combo box */
+  box2 = gtk_hbox_new (FALSE, 4);
   label = gtk_label_new_with_mnemonic (_("_Depth Color:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-  gtk_box_pack_start (GTK_BOX (depth_box), GTK_WIDGET (label), FALSE, FALSE, 0);
-
-  if (VINAGRE_IS_VNC_CONNECTION (conn))
-    depth_profile = vinagre_vnc_connection_get_depth_profile (VINAGRE_VNC_CONNECTION (conn));
-  else
-    depth_profile = 0;
+  gtk_box_pack_start (GTK_BOX (box2), GTK_WIDGET (label), FALSE, FALSE, 0);
 
   combo = gtk_combo_box_new_text ();
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Use Server Settings"));
@@ -412,43 +439,47 @@ impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("High Color (16 bits)"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Low Color (8 bits)"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Ultra Low Color (3 bits)"));
-  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), depth_profile);
+  gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
+			    has_conn ? vinagre_vnc_connection_get_depth_profile (VINAGRE_VNC_CONNECTION (conn))
+			    : vinagre_cache_prefs_get_integer ("vnc-connection", "depth-profile", 0));
   g_object_set_data (G_OBJECT (box), "depth_combo", combo);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
-  gtk_box_pack_start (GTK_BOX (depth_box), GTK_WIDGET (combo), FALSE, FALSE, 0);
-  gtk_table_attach_defaults (table, depth_box, 1, 2, 3, 4);
+  gtk_box_pack_start (GTK_BOX (box2), GTK_WIDGET (combo), FALSE, FALSE, 0);
+  gtk_table_attach_defaults (table, box2, 1, 2, 4, 5);
 
   /* SSH Tunneling */
-  ssh_box = gtk_hbox_new (FALSE, 4);
+  box2 = gtk_hbox_new (FALSE, 4);
 
   /* Translators: the whole sentence will be: Use host <hostname> as a SSH tunnel*/
   check = gtk_check_button_new_with_mnemonic (_("Use h_ost"));
   g_object_set_data (G_OBJECT (box), "use_ssh", check);
-  gtk_box_pack_start (GTK_BOX (ssh_box), check, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box2), check, FALSE, FALSE, 0);
 
   ssh_host_entry = gtk_entry_new ();
   gtk_widget_set_sensitive (ssh_host_entry, FALSE);
   g_object_set_data (G_OBJECT (box), "ssh_host", ssh_host_entry);
   /* Translators: This is the tooltip of the SSH tunneling entry */
   gtk_widget_set_tooltip_text (ssh_host_entry, _("hostname or user@hostname"));
-  gtk_box_pack_start (GTK_BOX (ssh_box), ssh_host_entry, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box2), ssh_host_entry, FALSE, FALSE, 0);
 
   /* Translators: the whole sentence will be: Use host <hostname> as a SSH tunnel*/
   label = gtk_label_new_with_mnemonic (_("as a SSH tunnel"));
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-  gtk_box_pack_start (GTK_BOX (ssh_box), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 0);
 
   g_signal_connect (check,
 		    "toggled",
 		    G_CALLBACK (ssh_tunnel_check_toggled_cb),
 		    box);
-  if (VINAGRE_IS_VNC_CONNECTION (conn))
-    ssh_host = vinagre_vnc_connection_get_ssh_tunnel_host (VINAGRE_VNC_CONNECTION (conn));
+
+  ssh_host = has_conn ? g_strdup (vinagre_vnc_connection_get_ssh_tunnel_host (VINAGRE_VNC_CONNECTION (conn)))
+                      : vinagre_cache_prefs_get_string  ("vnc-connection", "ssh-tunnel-host", NULL);
   if (ssh_host)
     gtk_entry_set_text (GTK_ENTRY (ssh_host_entry), ssh_host);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), ssh_host && *ssh_host);
+  g_free (ssh_host);
 
-  gtk_table_attach_defaults (table, ssh_box, 1, 2, 4, 5);
+  gtk_table_attach_defaults (table, box2, 1, 2, 5, 6);
 
   gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (table), FALSE, FALSE, 0);
   return box;
